@@ -18,8 +18,11 @@ considered internal, and *not* a public API.
 """
 import copy
 import logging
+import socket
 
+import botocore.exceptions
 import botocore.serialize
+import botocore.utils
 from botocore.signers import RequestSigner
 from botocore.config import Config
 from botocore.endpoint import EndpointCreator
@@ -28,14 +31,39 @@ from botocore.endpoint import EndpointCreator
 logger = logging.getLogger(__name__)
 
 
+VALID_STS_REGIONAL_ENDPOINTS_CONFIG = [
+    'legacy',
+    'regional',
+]
+LEGACY_GLOBAL_STS_REGIONS = [
+    'ap-northeast-1',
+    'ap-south-1',
+    'ap-southeast-1',
+    'ap-southeast-2',
+    'aws-global',
+    'ca-central-1',
+    'eu-central-1',
+    'eu-north-1',
+    'eu-west-1',
+    'eu-west-2',
+    'eu-west-3',
+    'sa-east-1',
+    'us-east-1',
+    'us-east-2',
+    'us-west-1',
+    'us-west-2',
+]
+
+
 class ClientArgsCreator(object):
     def __init__(self, event_emitter, user_agent, response_parser_factory,
-                 loader, exceptions_factory):
+                 loader, exceptions_factory, config_store):
         self._event_emitter = event_emitter
         self._user_agent = user_agent
         self._response_parser_factory = response_parser_factory
         self._loader = loader
         self._exceptions_factory = exceptions_factory
+        self._config_store = config_store
 
     def get_client_args(self, service_model, region_name, is_secure,
                         endpoint_url, verify, credentials, scoped_config,
@@ -51,6 +79,7 @@ class ClientArgsCreator(object):
         config_kwargs = final_args['config_kwargs']
         s3_config = final_args['s3_config']
         partition = endpoint_config['metadata'].get('partition', None)
+        socket_options = final_args['socket_options']
 
         signing_region = endpoint_config['signing_region']
         endpoint_region_name = endpoint_config['region_name']
@@ -77,7 +106,9 @@ class ClientArgsCreator(object):
             response_parser_factory=self._response_parser_factory,
             max_pool_connections=new_config.max_pool_connections,
             proxies=new_config.proxies,
-            timeout=(new_config.connect_timeout, new_config.read_timeout))
+            timeout=(new_config.connect_timeout, new_config.read_timeout),
+            socket_options=socket_options,
+            client_cert=new_config.client_cert)
 
         serializer = botocore.serialize.create_serializer(
             protocol, parameter_validation)
@@ -104,12 +135,9 @@ class ClientArgsCreator(object):
         if client_config and not client_config.parameter_validation:
             parameter_validation = False
         elif scoped_config:
-            raw_value = str(scoped_config.get('parameter_validation', ''))
-            if raw_value.lower() == 'false':
-                parameter_validation = False
-
-        endpoint_config = endpoint_bridge.resolve(
-            service_name, region_name, endpoint_url, is_secure)
+            raw_value = scoped_config.get('parameter_validation')
+            if raw_value is not None:
+                parameter_validation = botocore.utils.ensure_boolean(raw_value)
 
         # Override the user agent if specified in the client config.
         user_agent = self._user_agent
@@ -119,6 +147,13 @@ class ClientArgsCreator(object):
             if client_config.user_agent_extra is not None:
                 user_agent += ' %s' % client_config.user_agent_extra
 
+        endpoint_config = self._compute_endpoint_config(
+            service_name=service_name,
+            region_name=region_name,
+            endpoint_url=endpoint_url,
+            is_secure=is_secure,
+            endpoint_bridge=endpoint_bridge,
+        )
         # Create a new client config to be passed to the client based
         # on the final values. We do not want the user to be able
         # to try to modify an existing client with a client config.
@@ -132,7 +167,9 @@ class ClientArgsCreator(object):
                 read_timeout=client_config.read_timeout,
                 max_pool_connections=client_config.max_pool_connections,
                 proxies=client_config.proxies,
-                retries=client_config.retries
+                retries=client_config.retries,
+                client_cert=client_config.client_cert,
+                inject_host_prefix=client_config.inject_host_prefix,
             )
         s3_config = self.compute_s3_config(scoped_config,
                                            client_config)
@@ -144,6 +181,7 @@ class ClientArgsCreator(object):
             'protocol': protocol,
             'config_kwargs': config_kwargs,
             's3_config': s3_config,
+            'socket_options': self._compute_socket_options(scoped_config)
         }
 
     def compute_s3_config(self, scoped_config, client_config):
@@ -186,17 +224,49 @@ class ClientArgsCreator(object):
 
         return s3_configuration
 
+    def _compute_endpoint_config(self, service_name, region_name, endpoint_url,
+                                 is_secure, endpoint_bridge):
+        endpoint_config = endpoint_bridge.resolve(
+            service_name, region_name, endpoint_url, is_secure)
+        if self._should_set_global_sts_endpoint(
+                service_name, region_name, endpoint_url):
+            self._set_global_sts_endpoint(endpoint_config, is_secure)
+        return endpoint_config
+
+    def _should_set_global_sts_endpoint(self, service_name, region_name,
+                                        endpoint_url):
+        if service_name != 'sts':
+            return False
+        if endpoint_url:
+            return False
+        return (
+            self._get_sts_regional_endpoints_config() == 'legacy' and
+            region_name in LEGACY_GLOBAL_STS_REGIONS
+        )
+
+    def _get_sts_regional_endpoints_config(self):
+        sts_regional_endpoints_config = self._config_store.get_config_variable(
+            'sts_regional_endpoints')
+        if not sts_regional_endpoints_config:
+            sts_regional_endpoints_config = 'legacy'
+        if sts_regional_endpoints_config not in \
+                VALID_STS_REGIONAL_ENDPOINTS_CONFIG:
+            raise botocore.exceptions.InvalidSTSRegionalEndpointsConfigError(
+                sts_regional_endpoints_config=sts_regional_endpoints_config)
+        return sts_regional_endpoints_config
+
+    def _set_global_sts_endpoint(self, endpoint_config, is_secure):
+        scheme = 'https' if is_secure else 'http'
+        endpoint_config['endpoint_url'] = '%s://sts.amazonaws.com' % scheme
+        endpoint_config['signing_region'] = 'us-east-1'
+
     def _convert_config_to_bool(self, config_dict, keys):
         # Make sure any further modifications to this section of the config
         # will not affect the scoped config by making a copy of it.
         config_copy = config_dict.copy()
         present_keys = [k for k in keys if k in config_copy]
         for key in present_keys:
-            # Normalize on different possible values of True
-            if config_copy[key] in [True, 'True', 'true']:
-                config_copy[key] = True
-            else:
-                config_copy[key] = False
+            config_copy[key] = botocore.utils.ensure_boolean(config_copy[key])
         return config_copy
 
     def _get_default_s3_region(self, service_name, endpoint_bridge):
@@ -207,3 +277,20 @@ class ClientArgsCreator(object):
             endpoint = endpoint_bridge.resolve('s3')
             return endpoint['signing_region'], endpoint['region_name']
         return None, None
+
+    def _compute_socket_options(self, scoped_config):
+        # This disables Nagle's algorithm and is the default socket options
+        # in urllib3.
+        socket_options = [(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)]
+        if scoped_config:
+            # Enables TCP Keepalive if specified in shared config file.
+            if self._ensure_boolean(scoped_config.get('tcp_keepalive', False)):
+                socket_options.append(
+                    (socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1))
+        return socket_options
+
+    def _ensure_boolean(self, val):
+        if isinstance(val, bool):
+            return val
+        else:
+            return val.lower() == 'true'
