@@ -119,6 +119,7 @@ class Session(object):
         if profile is not None:
             self._session_instance_vars['profile'] = profile
         self._client_config = None
+        self._last_client_region_used = None
         self._components = ComponentLocator()
         self._internal_components = ComponentLocator()
         self._register_components()
@@ -141,8 +142,12 @@ class Session(object):
 
     def _register_credential_provider(self):
         self._components.lazy_register_component(
-            'credential_provider',
-            lambda:  botocore.credentials.create_credential_resolver(self))
+            'credential_provider', self._create_credential_resolver)
+
+    def _create_credential_resolver(self):
+        return botocore.credentials.create_credential_resolver(
+            self, region_name=self._last_client_region_used
+        )
 
     def _register_data_loader(self):
         self._components.lazy_register_component(
@@ -192,12 +197,13 @@ class Session(object):
     def _create_csm_monitor(self):
         if self.get_config_variable('csm_enabled'):
             client_id = self.get_config_variable('csm_client_id')
+            host = self.get_config_variable('csm_host')
             port = self.get_config_variable('csm_port')
             handler = monitoring.Monitor(
                 adapter=monitoring.MonitorEventAdapter(),
                 publisher=monitoring.SocketPublisher(
                     socket=socket.socket(socket.AF_INET, socket.SOCK_DGRAM),
-                    host='127.0.0.1',
+                    host=host,
                     port=port,
                     serializer=monitoring.CSMSerializer(
                         csm_client_id=client_id)
@@ -788,13 +794,7 @@ class Session(object):
         elif default_client_config is not None:
             config = default_client_config
 
-        # Figure out the user-provided region based on the various
-        # configuration options.
-        if region_name is None:
-            if config and config.region_name is not None:
-                region_name = config.region_name
-            else:
-                region_name = self.get_config_variable('region')
+        region_name = self._resolve_region_name(region_name, config)
 
         # Figure out the verify value base on the various
         # configuration options.
@@ -841,6 +841,26 @@ class Session(object):
             monitor.register(client.meta.events)
         return client
 
+    def _resolve_region_name(self, region_name, config):
+        # Figure out the user-provided region based on the various
+        # configuration options.
+        if region_name is None:
+            if config and config.region_name is not None:
+                region_name = config.region_name
+            else:
+                region_name = self.get_config_variable('region')
+        # For any client that we create in retrieving credentials
+        # we want to create it using the same region as specified in
+        # creating this client. It is important to note though that the
+        # credentials client is only created once per session. So if a new
+        # client is created with a different region, its credential resolver
+        # will use the region of the first client. However, that is not an
+        # issue as of now because the credential resolver uses only STS and
+        # the credentials returned at regional endpoints are valid across
+        # all regions in the partition.
+        self._last_client_region_used = region_name
+        return region_name
+
     def _missing_cred_vars(self, access_key, secret_key):
         if access_key is not None and secret_key is None:
             return 'aws_secret_access_key'
